package io.tswf.minecraft.blockventure.core.world.chunk.persist

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import it.unimi.dsi.fastutil.longs.LongSet
import net.minecraft.server.world.ServerWorld
import net.minecraft.util.math.ChunkPos

@Slf4j
@CompileStatic
class ChunkPersistService {

    static boolean shouldLogPersistOperations = System.findGlobalProperty("blockventure.world.chunk.persist.logs.enabled")
        .map { it.toBoolean() }
        .orElse(true)


    static void persistChunk(ServerWorld world, int chunkX, int chunkZ) {
        if (shouldLogPersistOperations && !isChunkPersisted(world, chunkX, chunkZ)) {
            log.info("Persisting chunk ${chunkX} ${chunkZ} in world ${world.dimensionKey.value}...")
        }

        world.setChunkForced(chunkX, chunkZ, true)
    }

    static void freeChunk(ServerWorld world, int chunkX, int chunkZ) {
        if (shouldLogPersistOperations && isChunkPersisted(world, chunkX, chunkZ)) {
            log.info("Free chunk ${chunkX} ${chunkZ} in world ${world.dimensionKey.value}...")
        }

        world.setChunkForced(chunkX, chunkZ, false)
    }

    static boolean isChunkPersisted(ServerWorld world, int chunkX, int chunkZ) {
        return isChunkPersisted(world, ChunkPos.toLong(chunkX, chunkZ))
    }

    static boolean isChunkPersisted(ServerWorld world, long serializedChunkPos) {
        def positions = getSerializedChunkPositions(world)
        return positions.contains(serializedChunkPos)
    }

    static LongSet getSerializedChunkPositions(ServerWorld serverWorld) {
        return serverWorld.forcedChunks
    }
}
